@@ -1,13 +1,19 @@
-use gl::types::*;
+extern crate gl;
+use core::mem;
+use core::ptr;
+use core::str;
+use fltk::{
+    app,
+    prelude::{WidgetExt, WindowExt},
+    window,
+};
+use gl::types::{GLchar, GLenum, GLint, GLsizeiptr, GLsync, GLuint};
 use std::ffi::CString;
-use std::mem;
 use std::os::raw::c_void;
-use std::ptr;
-use std::str;
 
 use egui::{
     paint::{Color32, Mesh, Texture},
-    vec2, ClippedMesh,
+    vec2, ClippedMesh, Pos2, Rect,
 };
 
 #[derive(Default)]
@@ -38,6 +44,7 @@ const VS_SRC: &str = r#"
     in vec2 a_tc;
     out vec4 v_rgba;
     out vec2 v_tc;
+
     // 0-1 linear  from  0-255 sRGB
     vec3 linear_from_srgb(vec3 srgb) {
         bvec3 cutoff = lessThan(srgb, vec3(10.31475));
@@ -45,9 +52,11 @@ const VS_SRC: &str = r#"
         vec3 higher = pow((srgb + vec3(14.025)) / vec3(269.025), vec3(2.4));
         return mix(higher, lower, cutoff);
     }
+
     vec4 linear_from_srgba(vec4 srgba) {
         return vec4(linear_from_srgb(srgba.rgb), srgba.a / 255.0);
     }
+
     void main() {
         gl_Position = vec4(
             2.0 * a_pos.x / u_screen_size.x - 1.0,
@@ -65,6 +74,7 @@ const FS_SRC: &str = r#"
     in vec4 v_rgba;
     in vec2 v_tc;
     out vec4 f_color;
+
     // 0-255 sRGB  from  0-1 linear
     vec3 srgb_from_linear(vec3 rgb) {
         bvec3 cutoff = lessThan(rgb, vec3(0.0031308));
@@ -72,6 +82,7 @@ const FS_SRC: &str = r#"
         vec3 higher = vec3(269.025) * pow(rgb, vec3(1.0 / 2.4)) - vec3(14.025);
         return mix(higher, lower, vec3(cutoff));
     }
+
     vec4 srgba_from_linear(vec4 rgba) {
         return vec4(srgb_from_linear(rgba.rgb), 255.0 * rgba.a);
     }
@@ -82,9 +93,11 @@ const FS_SRC: &str = r#"
         vec3 higher = pow((srgb + vec3(14.025)) / vec3(269.025), vec3(2.4));
         return mix(higher, lower, vec3(cutoff));
     }
+
     vec4 linear_from_srgba(vec4 srgba) {
         return vec4(linear_from_srgb(srgba.rgb), srgba.a / 255.0);
     }
+
     void main() {
         // Need to convert from SRGBA to linear.
         vec4 texture_rgba = linear_from_srgba(texture(u_sampler, v_tc) * 255.0);
@@ -99,13 +112,15 @@ pub struct Painter {
     pos_buffer: GLuint,
     tc_buffer: GLuint,
     color_buffer: GLuint,
-    canvas_width: u32,
-    canvas_height: u32,
     egui_texture: GLuint,
+    // Call fence for sdl2 vsync so the CPU won't heat up if there's no heavy activity.
+    // not sure in fltk GlutWindow, enabled by default?.
+    pub gl_sync_fence: GLsync,
     egui_texture_version: Option<u64>,
-    vert_shader: GLuint,
-    frag_shader: GLuint,
     user_textures: Vec<UserTexture>,
+    pub pixels_per_point: f32,
+    pub canvas_size: (i32, i32),
+    pub screen_rect: Rect,
 }
 
 pub fn compile_shader(src: &str, ty: GLenum) -> GLuint {
@@ -174,15 +189,11 @@ pub fn link_program(vs: GLuint, fs: GLuint) -> GLuint {
 }
 
 impl Painter {
-    pub fn new(
-        fl_glutwin: &fltk::window::GlutWindow,
-        canvas_width: u32,
-        canvas_height: u32,
-    ) -> Painter {
+    pub fn new(window: &mut window::GlutWindow, scale: f32) -> Painter {
         unsafe {
             let mut egui_texture = 0;
-            fltk::app::set_screen_scale(0, 1.);
-            gl::load_with(|name| fl_glutwin.get_proc_address(name) as *const _);
+            app::set_screen_scale(window.screen_num(), 1.);
+            gl::load_with(|name| window.get_proc_address(name) as *const _);
             gl::GenTextures(1, &mut egui_texture);
             gl::BindTexture(gl::TEXTURE_2D, egui_texture);
             gl::TexParameteri(gl::TEXTURE_2D, gl::TEXTURE_WRAP_S, gl::CLAMP_TO_EDGE as i32);
@@ -206,24 +217,41 @@ impl Painter {
             gl::GenBuffers(1, &mut tc_buffer);
             gl::GenBuffers(1, &mut color_buffer);
 
+            let (width, height) = (window.width(), window.height());
+            let pixels_per_point = scale;
+            let rect = vec2(width as f32, height as f32) / pixels_per_point;
+            let screen_rect = Rect::from_min_size(Pos2::new(0f32, 0f32), rect);
+
+            gl::DetachShader(program, vert_shader);
+            gl::DetachShader(program, frag_shader);
+            gl::DeleteShader(vert_shader);
+            gl::DeleteShader(frag_shader);
+
             Painter {
                 vertex_array,
                 program,
-                canvas_width,
-                canvas_height,
                 index_buffer,
                 pos_buffer,
                 tc_buffer,
                 color_buffer,
                 egui_texture,
-                vert_shader,
-                frag_shader,
+                gl_sync_fence: gl::FenceSync(gl::SYNC_GPU_COMMANDS_COMPLETE, 0),
+                pixels_per_point,
                 egui_texture_version: None,
                 user_textures: Default::default(),
+                canvas_size: (width, height),
+                screen_rect,
             }
         }
     }
 
+    pub fn update_screen_rect(&mut self, size: (i32, i32)) {
+        self.canvas_size = size;
+        let (x, y) = size;
+        let rect = vec2(x as f32, y as f32) / self.pixels_per_point;
+        self.screen_rect = Rect::from_min_size(Default::default(), rect);
+    }
+
     pub fn new_user_texture(
         &mut self,
         size: (usize, usize),
@@ -234,10 +262,10 @@ impl Painter {
 
         let mut pixels: Vec<u8> = Vec::with_capacity(srgba_pixels.len() * 4);
         for srgba in srgba_pixels {
-            pixels.push(srgba.r());
-            pixels.push(srgba.g());
-            pixels.push(srgba.b());
-            pixels.push(srgba.a());
+            pixels.push(srgba[0]);
+            pixels.push(srgba[1]);
+            pixels.push(srgba[2]);
+            pixels.push(srgba[3]);
         }
 
         let id = egui::TextureId::User(self.user_textures.len() as u64);
@@ -259,10 +287,10 @@ impl Painter {
         let mut pixels: Vec<u8> = Vec::with_capacity(texture.pixels.len() * 4);
         for &alpha in &texture.pixels {
             let srgba = Color32::from_white_alpha(alpha);
-            pixels.push(srgba.r());
-            pixels.push(srgba.g());
-            pixels.push(srgba.b());
-            pixels.push(srgba.a());
+            pixels.push(srgba[0]);
+            pixels.push(srgba[1]);
+            pixels.push(srgba[2]);
+            pixels.push(srgba[3]);
         }
 
         unsafe {
@@ -292,7 +320,7 @@ impl Painter {
     fn upload_user_textures(&mut self) {
         unsafe {
             for user_texture in &mut self.user_textures {
-                if user_texture.texture.is_some() && !user_texture.dirty {
+                if !user_texture.texture.is_none() && !user_texture.dirty {
                     continue;
                 }
                 let pixels = std::mem::take(&mut user_texture.pixels);
@@ -376,10 +404,10 @@ impl Painter {
                 self.user_textures[id].pixels = Vec::with_capacity(pixels.len() * 4);
 
                 for p in pixels {
-                    self.user_textures[id].pixels.push(p.r());
-                    self.user_textures[id].pixels.push(p.g());
-                    self.user_textures[id].pixels.push(p.b());
-                    self.user_textures[id].pixels.push(p.a());
+                    self.user_textures[id].pixels.push(p[0]);
+                    self.user_textures[id].pixels.push(p[1]);
+                    self.user_textures[id].pixels.push(p[2]);
+                    self.user_textures[id].pixels.push(p[3]);
                 }
                 self.user_textures[id].dirty = true;
             }
@@ -391,11 +419,12 @@ impl Painter {
         bg_color: Option<Color32>,
         meshes: Vec<ClippedMesh>,
         egui_texture: &Texture,
-        pixels_per_point: f32,
     ) {
         self.upload_egui_texture(egui_texture);
         self.upload_user_textures();
 
+        let (canvas_width, canvas_height) = self.canvas_size;
+        let pixels_per_point = self.pixels_per_point;
         unsafe {
             if let Some(color) = bg_color {
                 gl::ClearColor(
@@ -411,7 +440,6 @@ impl Painter {
             //can do the blending correctly. Not setting the framebuffer
             //leads to darkened, oversaturated colors.
             gl::Enable(gl::FRAMEBUFFER_SRGB);
-
             gl::Enable(gl::SCISSOR_TEST);
             gl::Enable(gl::BLEND);
             gl::BlendFunc(gl::ONE, gl::ONE_MINUS_SRC_ALPHA); // premultiplied alpha
@@ -421,7 +449,7 @@ impl Painter {
             let u_screen_size = CString::new("u_screen_size").unwrap();
             let u_screen_size_ptr = u_screen_size.as_ptr();
             let u_screen_size_loc = gl::GetUniformLocation(self.program, u_screen_size_ptr);
-            let screen_size_pixels = vec2(self.canvas_width as f32, self.canvas_height as f32);
+            let screen_size_pixels = vec2(canvas_width as f32, canvas_height as f32);
             let screen_size_points = screen_size_pixels / pixels_per_point;
             gl::Uniform2f(
                 u_screen_size_loc,
@@ -432,7 +460,7 @@ impl Painter {
             let u_sampler_ptr = u_sampler.as_ptr();
             let u_sampler_loc = gl::GetUniformLocation(self.program, u_sampler_ptr);
             gl::Uniform1i(u_sampler_loc, 0);
-            gl::Viewport(0, 0, self.canvas_width as i32, self.canvas_height as i32);
+            gl::Viewport(0, 0, canvas_width as i32, canvas_height as i32);
 
             for ClippedMesh(clip_rect, mesh) in meshes {
                 gl::BindTexture(gl::TEXTURE_2D, self.get_texture(mesh.texture_id));
@@ -453,64 +481,54 @@ impl Painter {
                 //scissor Y coordinate is from the bottom
                 gl::Scissor(
                     clip_min_x,
-                    self.canvas_height as i32 - clip_max_y,
+                    canvas_height as i32 - clip_max_y,
                     clip_max_x - clip_min_x,
                     clip_max_y - clip_min_y,
                 );
 
                 self.paint_mesh(&mesh);
-                gl::Disable(gl::SCISSOR_TEST);
             }
-            gl::Disable(gl::FRAMEBUFFER_SRGB);
-        }
-    }
 
-    pub fn cleanup(&self) {
-        unsafe {
-            gl::DeleteProgram(self.program);
-            gl::DeleteShader(self.vert_shader);
-            gl::DeleteShader(self.frag_shader);
-            gl::DeleteBuffers(1, &self.pos_buffer);
-            gl::DeleteBuffers(1, &self.tc_buffer);
-            gl::DeleteBuffers(1, &self.color_buffer);
-            gl::DeleteBuffers(1, &self.index_buffer);
-            gl::DeleteVertexArrays(1, &self.vertex_array);
+            gl::Disable(gl::SCISSOR_TEST);
+            gl::Disable(gl::FRAMEBUFFER_SRGB);
         }
     }
 
     fn paint_mesh(&self, mesh: &Mesh) {
         debug_assert!(mesh.is_valid());
-        let indices: Vec<u16> = mesh.indices.iter().map(|idx| *idx as u16).collect();
-
-        let mut positions: Vec<f32> = Vec::with_capacity(2 * mesh.vertices.len());
-        let mut tex_coords: Vec<f32> = Vec::with_capacity(2 * mesh.vertices.len());
-        for v in &mesh.vertices {
-            positions.push(v.pos.x);
-            positions.push(v.pos.y);
-            tex_coords.push(v.uv.x);
-            tex_coords.push(v.uv.y);
-        }
-
-        let mut colors: Vec<u8> = Vec::with_capacity(4 * mesh.vertices.len());
-        for v in &mesh.vertices {
-            colors.push(v.color[0]);
-            colors.push(v.color[1]);
-            colors.push(v.color[2]);
-            colors.push(v.color[3]);
-        }
-
         unsafe {
+            let indices: Vec<u16> = mesh.indices.iter().map(move |idx| *idx as u16).collect();
+            let indices_len = indices.len();
+            let vertices = &mesh.vertices;
+            let vertices_len = vertices.len();
+
+            // --------------------------------------------------------------------
+
             gl::BindVertexArray(self.vertex_array);
             gl::BindBuffer(gl::ELEMENT_ARRAY_BUFFER, self.index_buffer);
             gl::BufferData(
                 gl::ELEMENT_ARRAY_BUFFER,
-                (indices.len() * mem::size_of::<u16>()) as GLsizeiptr,
+                (indices_len * mem::size_of::<u16>()) as GLsizeiptr,
                 //mem::transmute(&indices.as_ptr()),
                 indices.as_ptr() as *const gl::types::GLvoid,
                 gl::STREAM_DRAW,
             );
 
             // --------------------------------------------------------------------
+
+            let mut positions: Vec<f32> = Vec::with_capacity(2 * vertices_len);
+            let mut tex_coords: Vec<f32> = Vec::with_capacity(2 * vertices_len);
+            {
+                for v in &mesh.vertices {
+                    positions.push(v.pos.x);
+                    positions.push(v.pos.y);
+                    tex_coords.push(v.uv.x);
+                    tex_coords.push(v.uv.y);
+                }
+            }
+
+            // --------------------------------------------------------------------
+
             gl::BindBuffer(gl::ARRAY_BUFFER, self.pos_buffer);
             gl::BufferData(
                 gl::ARRAY_BUFFER,
@@ -552,6 +570,17 @@ impl Painter {
             gl::EnableVertexAttribArray(a_tc_loc);
 
             // --------------------------------------------------------------------
+
+            let mut colors: Vec<u8> = Vec::with_capacity(4 * vertices_len);
+            {
+                for v in vertices {
+                    colors.push(v.color[0]);
+                    colors.push(v.color[1]);
+                    colors.push(v.color[2]);
+                    colors.push(v.color[3]);
+                }
+            }
+
             gl::BindBuffer(gl::ARRAY_BUFFER, self.color_buffer);
             gl::BufferData(
                 gl::ARRAY_BUFFER,
@@ -582,12 +611,24 @@ impl Painter {
 
             gl::DrawElements(
                 gl::TRIANGLES,
-                indices.len() as i32,
+                indices_len as i32,
                 gl::UNSIGNED_SHORT,
                 ptr::null(),
             );
         }
     }
+
+    pub fn cleanup(&self) {
+        unsafe {
+            gl::DeleteSync(self.gl_sync_fence);
+            gl::DeleteProgram(self.program);
+            gl::DeleteBuffers(1, &self.pos_buffer);
+            gl::DeleteBuffers(1, &self.tc_buffer);
+            gl::DeleteBuffers(1, &self.color_buffer);
+            gl::DeleteBuffers(1, &self.index_buffer);
+            gl::DeleteVertexArrays(1, &self.vertex_array);
+        }
+    }
 }
 
 impl Drop for Painter {
